#!/bin/sh
#########################################################################
# Shell script to scan for astropersons, astroplaces, and astropostal
# Volker Ossenkopf, Jena - 12/2/1999
#########################################################################

# Path of all executables needed in the script: 
# sed, awk, cgiparse, agrep have to be found
PATH=/bin:/usr/bin:/usr/lib/cgi-bin

# Variables for the different files
# NAME?   = Description in the HTML-Form
# DIRSUB? = Local file name

# First file
NAME1='persons'
DIRSUB1=/home/www-data/astronomy/astropersons.lis
# Seconf file
NAME2='places'
DIRSUB2=/home/www-data/astronomy/astroplaces.lis
# Third file
NAME3='postal'
DIRSUB3=/home/www-data/astronomy/astropostal.lis

# The name and location of the script itself
ACCESS="http://chandra.ph1.uni-koeln.de/cgi-bin/astrosearch.sh"

# Name of the System V/R4 awk command: in general awk or gawk, 
# but on SUN-OS and Solaris: nawk
AWK=gawk

#########################################################################
# No changes should be necessary below this line
#########################################################################

# Produce header
echo 'Content-type: text/html'
echo ''

# Extract parameters from QUERY_STRING
eval `cgiparse -form`

# Test for cgiparse and reasonable input
if [ "${FORM_words}" != 'substring' -a "${FORM_words}" != 'whole words' ]
then 
echo "Cgiparse error in processing the query string.<p>"
echo "Please, check the installation of cgiparse and the input form."
exit 1
fi

# HTML Header
cat << !!
<HTML> <HEAD>
<TITLE>Search results</TITLE>
</HEAD> <BODY>
<H1 align=center>Search results for:   
!!

# Options for agrep 
ENO='0'`echo "${FORM_fuzzy}" | sed -e 's/[^0-9]//g'`
PAR="-${ENO}"

case "$FORM_cases" in
ignore) PAR=$PAR' -i';;
esac

case "$FORM_words" in
whole?words) PAR=$PAR' -w';;
esac

# Files
# The astropostal file has to be converted into one-line form here.
case "${FORM_baum}" in
"${NAME1}") 	FNAME='persons'
		DIRSUB=$DIRSUB1 ;;
"${NAME2}") 	FNAME='places'
		DIRSUB=$DIRSUB2 ;;
*) 		FNAME='postal'
		DIRSUB='-e /^[A-Za-z]/N -e s/\n/:/ '$DIRSUB3 ;;
esac

# Print search expression in the header
WEXPR=`echo "${FORM_suchwort}" |sed 's/ *$//'` 
EXPR=`echo $WEXPR |sed -e 's/[)(,;.]/\\\\&/g' -e 's/  */;/g'`
if [ "${EXPR}" = '' ];  then EXPR='complete_nonsense'; fi
echo "${WEXPR} </h1><p>"

# Find the expression
sed -e '1,/^\*/d' $DIRSUB |\
agrep $PAR "${EXPR}" 2>/dev/null |\
agrep -v 'agrep_' 2>/dev/null |\

# Formatting using awk
$AWK 'BEGIN { 	list1[1]="a"; list2[1]="Jan 88";
		list1[2]="b"; list2[2]="Jul 88";
		list1[3]="c"; list2[3]="Jan 89";
		list1[4]="d"; list2[4]="Jul 89";
		list1[5]="e"; list2[5]="Jan 90";
		list1[6]="f"; list2[6]="Jul 90";
		list1[7]="g"; list2[7]="Jan 91";
		list1[8]="h"; list2[8]="Jul 91";
		list1[9]="i"; list2[9]="Jan 92";
		list1[10]="j"; list2[10]="Jul 92";
		list1[11]="k"; list2[11]="Jan 93";
		list1[12]="l"; list2[12]="Apr 94";
		list1[13]="m"; list2[13]="May 95";
		list1[14]="n"; list2[14]="Jul 96";
		list1[15]="6"; list2[15]="1996";
		list1[16]="7"; list2[16]="1997";
		list1[17]="8"; list2[17]="1998";
		list1[18]="9"; list2[18]="1999";
	 	lista1[1]="B"; lista2[1]="BITNET";
		lista1[2]="D"; lista2[2]="DECNET";
		lista1[3]="I"; lista2[3]="INTERNET";
		lista1[4]="U"; lista2[4]="UUCP";  
		HHOLD=""; NUM=0 }
	function persons() {
		HH = substr($0, 1,23); ABB = substr($0, 24,9); NET = substr($0, 33,1);
		EML = substr($0, 35,42); DT = substr($0, 77,2);
		gsub(" +$","",HH); gsub(" +$","",DT); gsub(" ","",ABB); 
		gsub("^[0-9]","19&",DT);
		for ( i=1; i<5; i++) gsub("^"lista1[i],lista2[i],NET);
    		printf("<hr><h3>%s</h3>\n", HH);
    		printf("Institute: <a href=\"'${ACCESS}'?suchwort=%s&baum=places&fuzzy=0&words=whole+words\"> %s</a><br>", ABB, ABB);
		printf("E-Mail: %s (%s)<p>\n", EML, NET);
    		if (DT != "") printf("<i>Last updated: %s</i><p>\n", DT);
		NUM++ } 
	function places() {
		HH = substr($0, 1,32); ABB = substr($0, 33,8); NET = substr($0, 41,37);
		DT = substr($0, 78,1);
		gsub(" +$","",HH); gsub(" +$","",DT); gsub(" +$","",ABB);
		gsub(" 9$","",NET);
		gsub("http:[^ ]*","<a href=\"&\">&</a>",NET);
		for ( i=1; i<19; i++) gsub("^"list1[i],list2[i],DT);
		if (HH != HHOLD) { 
		    if (ABB == "") printf("<hr><h3>%s</h3>\n", HH);
		    else {
			HHPR = HH; gsub(" ","+",HHPR); gsub("\"","%22",HHPR); gsub("&","%26",HHPR);
			printf("<hr><h3><a href=\"'${ACCESS}'?suchwort=%s&baum=postal&fuzzy=0&words=substring\"> %s</a> (%s)</h3>\n", HHPR, HH, ABB); 
    			if (DT != "") printf("<i>Last updated: %s</i><p>\n", DT); }
		    NUM++ }
		printf("%s <p>\n",NET);
		HHOLD=HH } 
	function postal() {
		split($0, NOM, ":");
		printf("<hr><h3>%s</h3>\n",NOM[1]);
		printf("%s<p>\n", NOM[2]);
		NUM++ }
	{ '${FNAME}'() }
   END 	{  if ( NUM == 0 ) { printf("<hr><p><b>No entries found</b><p><hr>\n");
		printf("<h2 align=center>Search for similar strings:</h2>\n");
		exit 1 }
		printf("<hr><h2 align=center>Total: %d matches</h2><hr>\n",NUM) } 
   '

# Make additional runs if no entries were found
FOUND=$?
while [ '(' $FOUND -ne 0 ')' -a  '(' $ENO -le 3 ')' ] 
do
PAR="-i -$ENO"
ENO=`expr "${ENO}" + 1`

# Here, we have a simple copy of all of the above
sed -e '1,/^\*/d' $DIRSUB |\
agrep $PAR "${EXPR}" 2>/dev/null |\
agrep -v 'agrep_' 2>/dev/null |\

# Formatting using awk
$AWK 'BEGIN { 	list1[1]="a"; list2[1]="Jan 88";
		list1[2]="b"; list2[2]="Jul 88";
		list1[3]="c"; list2[3]="Jan 89";
		list1[4]="d"; list2[4]="Jul 89";
		list1[5]="e"; list2[5]="Jan 90";
		list1[6]="f"; list2[6]="Jul 90";
		list1[7]="g"; list2[7]="Jan 91";
		list1[8]="h"; list2[8]="Jul 91";
		list1[9]="i"; list2[9]="Jan 92";
		list1[10]="j"; list2[10]="Jul 92";
		list1[11]="k"; list2[11]="Jan 93";
		list1[12]="l"; list2[12]="Apr 94";
		list1[13]="m"; list2[13]="May 95";
		list1[14]="n"; list2[14]="Jul 96";
		list1[15]="6"; list2[15]="1996";
		list1[16]="7"; list2[16]="1997";
		list1[17]="8"; list2[17]="1998";
		list1[18]="9"; list2[18]="1999";
	 	lista1[1]="B"; lista2[1]="BITNET";
		lista1[2]="D"; lista2[2]="DECNET";
		lista1[3]="I"; lista2[3]="INTERNET";
		lista1[4]="U"; lista2[4]="UUCP";  
		HHOLD=""; NUM=0 }
	function persons() {
		HH = substr($0, 1,23); ABB = substr($0, 24,9); NET = substr($0, 33,1);
		EML = substr($0, 35,42); DT = substr($0, 77,2);
		gsub(" +$","",HH); gsub(" +$","",DT); gsub(" ","",ABB); 
		gsub("^[0-9]","19&",DT);
		for ( i=1; i<5; i++) gsub("^"lista1[i],lista2[i],NET);
    		printf("<hr><h3>%s</h3>\n", HH);
    		printf("Institute: <a href=\"'${ACCESS}'?suchwort=%s&baum=places&fuzzy=0&words=whole+words\"> %s</a><br>", ABB, ABB);
		printf("E-Mail: %s (%s)<p>\n", EML, NET);
    		if (DT != "") printf("<i>Last updated: %s</i><p>\n", DT);
		NUM++ } 
	function places() {
		HH = substr($0, 1,32); ABB = substr($0, 33,8); NET = substr($0, 41,37);
		DT = substr($0, 78,1);
		gsub(" +$","",HH); gsub(" +$","",DT); gsub(" +$","",ABB);
		gsub(" 9$","",NET);
		gsub("http:[^ ]*","<a href=\"&\">&</a>",NET);
		for ( i=1; i<19; i++) gsub("^"list1[i],list2[i],DT);
		if (HH != HHOLD) { 
		    if (ABB == "") printf("<hr><h3>%s</h3>\n", HH);
		    else {
			HHPR = HH; gsub(" ","+",HHPR); gsub("\"","%22",HHPR); gsub("&","%26",HHPR);
			printf("<hr><h3><a href=\"'${ACCESS}'?suchwort=%s&baum=postal&fuzzy=0&words=substring\"> %s</a> (%s)</h3>\n", HHPR, HH, ABB); 
    			if (DT != "") printf("<i>Last updated: %s</i><p>\n", DT); }
		    NUM++ }
		printf("%s <p>\n",NET);
		HHOLD=HH } 
	function postal() {
		split($0, NOM, ":");
		printf("<hr><h3>%s</h3>\n",NOM[1]);
		printf("%s<p>\n", NOM[2]);
		NUM++ }
	{ '${FNAME}'() }
   END 	{  if ( NUM == 0 ) exit 1
		printf("<hr><h2 align=center>Total: %d matches</h2><hr>\n",NUM) } 
   '
FOUND=$?
done

# Final output if no entries were found
if [ $FOUND -ne 0 ]
then
echo "<hr><p><b>No entries found.</b><p><hr>"
echo "<h2 align=center>Total: 0 matches</h2><hr>"
fi

#  Control output of input parameters 
echo "<strong>Search parameters:</strong> File: astro${FNAME}.lis, Cases: ${FORM_cases}, Matching: ${FORM_words}, ${FORM_fuzzy} errors.<hr><p>"

echo "</BODY></HTML>"
#  That's all

